
math.randomseed( os.time() )


-- Helper Functions

function collideCheck(x1, y1, w1, h1, x2, y2, w2, h2) -- the x, y, w, and h values of objects 1 and 2
	local collided = 0
	if (x1 > x2 and x1 < x2 + w2) or (x1 + w1 > x2 and x1 + w1 < x2 + w2) then -- in other words
		if (y1 > y2 and y1 < y2 + h2) or (y1 + h1 > y2 and y1 + h1 < y2 + h2) then
			collided = 1
		end
	end
	return collided
end




function bounceDown(angle)
	angle = (2 * math.pi) - angle
	return angle
end

function bounceRight(angle)
	angle = math.pi - angle
	return angle
end

function bounceUp(angle)
	angle = (2 * math.pi) - angle
	return angle
end

function bounceLeft(angle)
	angle = math.pi - angle
	return angle
end


function drawBat()
	love.graphics.setColor( 100, 220, 160 )
	love.graphics.rectangle("fill", Bat.x, Bat.y, Bat.width, Bat.height)
end 

function updateBat(dt)
	if love.keyboard.isDown("down") then
		Bat.y = Bat.y + (Bat.speed * dt)
	elseif love.keyboard.isDown("up") then
		Bat.y = Bat.y - (Bat.speed * dt)
	end
end


function updateFoe(dt)
	local bCentre = Ball.y + (Ball.height / 2)
	local fCentre = Foe.y + (Foe.height / 2)
	if bCentre < fCentre then 
		Foe.y = Foe.y - (Foe.speed * dt)
	end
	if bCentre > fCentre then
		Foe.y = Foe.y + (Foe.speed * dt)
	end
	
	if not (love.keyboard.isDown("down") or love.keyboard.isDown("up")) then
		shrinkFoe(dt)
	else
		growFoe(dt)
	end
end


function shrinkFoe(dt)
	if Foe.height > 10 then
		Foe.height = Foe.height - (Foe.shrinkRate * dt)
	end
end

function growFoe(dt)
	Foe.height = Foe.height + (Foe.growRate * dt)
end




function drawFoe()
	love.graphics.setColor( 0, 220, 30 )
	love.graphics.rectangle("line", Foe.x, Foe.y, Foe.width, Foe.height)
end


function drawBall()
	love.graphics.setColor( 180, 20, 60 )
	love.graphics.rectangle("fill", Ball.x, Ball.y, Ball.width, Ball.height)
end

function updateBall(dt)
	Ball.x = Ball.x + math.cos(Ball.angle) * Ball.speed * dt
	Ball.y = Ball.y + math.sin(Ball.angle) * Ball.speed * dt
	
	if love.keyboard.isDown("l") then
		Ball.angle = Ball.angle + 0.01
	end
	
	if Ball.angle > 2 * math.pi then
		Ball.angle = Ball.angle - (2 * math.pi)
	end
	
	if Ball.angle < 0 then
		Ball.angle = Ball.angle + (2 * math.pi)
	end
	
	if Ball.y < 0 then
		Ball.angle = bounceDown(Ball.angle)
		Ball.y = Ball.y + 1
	end
	
	if Ball.x < 0 then
		Ball.angle = bounceRight(Ball.angle)
		Ball.x = Ball.x + 1
		Foe.score = Foe.score + 1
	end
	
	if Ball.y + Ball.height > ScreenHeight then
		Ball.angle = bounceUp(Ball.angle)
		Ball.y = Ball.y - 1
	end
	
	if Ball.x + Ball.width > ScreenWidth then
		Ball.angle = bounceLeft(Ball.angle)
		Ball.x = Ball.x - 1
		Bat.score = Bat.score + 1
	end
	
	if collideCheck(Ball.x, Ball.y, Ball.width, Ball.height, Bat.x, Bat.y, Bat.width, Bat.height) == 1 then
		Ball.angle = bounceRight(Ball.angle)
		Ball.speed = Ball.speed + 1
		Ball.x = Ball.x + 1
		
		love.audio.play(Blup)
		
		local bCentre = Ball.y + (Ball.height / 2)
		local tCentre = Bat.y + (Bat.height / 2)
		
		if bCentre < (tCentre - 5) then
			Ball.angle = Ball.angle - (math.random() / 3)
		elseif bCentre > (tCentre + 5) then 
			Ball.angle = Ball.angle + (math.random() / 3)
		end
		
	end
	
	if collideCheck(Ball.x, Ball.y, Ball.width, Ball.height, Foe.x, Foe.y, Foe.width, Foe.height) == 1 then
		Ball.angle = bounceLeft(Ball.angle)
		Ball.speed = Ball.speed + 1
		Ball.x = Ball.x - 1
		
		love.audio.play(Blop)
		
	end
	
	
end




function drawHUD()
	love.graphics.setColor( 255, 255, 255 )
	-- love.graphics.print (Ball.angle, 10, 10, 0, 1, 1)
	local message = "            Player: " .. Bat.score .. "            Enemy: " .. Foe.score
	love.graphics.print (message, 185, 10, 0, 1, 1)
end


-- Callback Functions

function love.load()
	ScreenWidth = 640
	ScreenHeight = 480
	
	love.graphics.setMode( ScreenWidth, ScreenHeight, false, false, 0 )
	love.graphics.setCaption( "Pong, with Love!" )

	Bat = {}
	Ball = {}
	Foe = {}
	
	Foe.x = 600
	Foe.y = 50
	Foe.speed = 100
	Foe.width = 20
	Foe.height = 50
	Foe.growRate = 10
	Foe.shrinkRate = 1
	Foe.score = 0

	Bat.x = 30
	Bat.y = 80
	Bat.speed = 100
	Bat.width = 20
	Bat.height = 50
	Bat.score = 0
	
	
	Ball.x = 100
	Ball.y = 100
	Ball.speed = 160
	Ball.width = 10
	Ball.height = 10
	Ball.angle = 0.2
	
	Blop = love.audio.newSource("blop.wav", "static")
	Blup = love.audio.newSource("blop.wav", "static")

	love.graphics.setBackgroundColor( 0, 0, 0 )

end

function love.update(dt)

	updateBat(dt)
	updateBall(dt)
	updateFoe(dt)

end

function love.draw()
	drawBat()
	drawBall()
	drawFoe()
	drawHUD()
end
